package cn.iocoder.yudao.module.member.controller.admin.userScore;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.member.api.score.MemberUserScoreApiImpl;
import cn.iocoder.yudao.module.member.api.score.dto.MemberUserScoreBatchOperateReqDTO;
import cn.iocoder.yudao.module.member.api.score.dto.MemberUserScoreOperateRespDTO;
import cn.iocoder.yudao.module.member.enums.ScoreOperateTypeEnum;
import cn.iocoder.yudao.module.member.enums.ScoreSourceTypeEnum;
import cn.iocoder.yudao.module.member.service.score.MemberUserScoreService;
import cn.iocoder.yudao.module.member.vo.memberUserScore.MemberUserScoreBackVO;
import cn.iocoder.yudao.module.member.vo.memberUserScore.MemberUserScoreOperateQueryVO;
import cn.iocoder.yudao.module.member.vo.memberUserScore.MemberUserScoreQueryVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Validated
@RestController
@Api(tags = "管理后台 - 会员积分")
@RequestMapping("/member/user-score")
public class MemberUserScoreController {

    @Resource
    private MemberUserScoreService userScoreService;
    @Resource
    private MemberUserScoreApiImpl scoreApi;


    @PostMapping("/page")
    @ApiOperation("获得会员积分分页")
    @PreAuthorize("@ss.hasPermission('member:user-score:query')")
    public CommonResult<PageResult<MemberUserScoreBackVO>> getUserScorePage(@Valid @RequestBody MemberUserScoreQueryVO query) {
        PageResult<MemberUserScoreBackVO> pageResult = userScoreService.getUserScorePage(query);
        return success(pageResult);
    }

    @PostMapping("/operate")
    @ApiOperation("操作积分")
    @PreAuthorize("@ss.hasPermission('member:user-score:operate')")
    public CommonResult<Boolean> operate(@Valid @RequestBody MemberUserScoreOperateQueryVO query) {
        Map<String,Object> extParam = new HashMap<>();
        extParam.put("comment", query.getComment());
        List<MemberUserScoreOperateRespDTO> memberUserScoreOperateRespDTOS = scoreApi.batchOperateScore(MemberUserScoreBatchOperateReqDTO.builder()
                .memberIds(query.getMemberIds())
                .scoreCount(query.getScoreCount())
                .operateType(ScoreOperateTypeEnum.parseByValue(query.getOperateType()))
                .sourceType(ScoreSourceTypeEnum.MANUAL_OPERATE)
                .extParam(extParam)
                .uniqueId(String.valueOf(System.currentTimeMillis()))
                .build());
        return success(null);
    }
}
