package cn.iocoder.yudao.module.member.service.level;

import cn.hutool.core.bean.BeanUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.member.dal.dataobject.level.MemberUserLevelConfigDO;
import cn.iocoder.yudao.module.member.dal.mysql.memberUserLevel.MemberUserLevelConfigMapper;
import cn.iocoder.yudao.module.member.vo.userLevel.MemberUserLevelConfigBaseRespVO;
import cn.iocoder.yudao.module.member.vo.userLevel.MemberUserLevelConfigReqVO;
import cn.iocoder.yudao.module.member.vo.userLevel.MemberUserLevelConfigUpdateVO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.member.enums.ErrorCodeConstants.LEVEL_BOUND_RANGE_CONFLICT;
import static cn.iocoder.yudao.module.member.enums.ErrorCodeConstants.LEVEL_BOUND_RANGE_ERROR;

/**
 * 会员积分 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
@Lazy
public class MemberUserLevelConfigServiceImpl extends AbstractService<MemberUserLevelConfigMapper, MemberUserLevelConfigDO> implements MemberUserLevelConfigService {

    private final MemberUserLevelConfigMapper memberUserLevelConfigMapper;

    public MemberUserLevelConfigServiceImpl(MemberUserLevelConfigMapper memberUserLevelConfigMapper) {
        this.memberUserLevelConfigMapper = memberUserLevelConfigMapper;
    }

    @Override
    public PageResult<MemberUserLevelConfigBaseRespVO> getLevelPage(MemberUserLevelConfigReqVO query) {
        int start = (query.getPageNo() - 1) * query.getPageSize();
        int size = query.getPageSize();
        LambdaQueryWrapper<MemberUserLevelConfigDO> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(MemberUserLevelConfigDO::getDeleted, 0);
        wrapper.last(String.format("limit %s, %s", start, size));
        List<MemberUserLevelConfigDO> list = this.list(wrapper);
        List<MemberUserLevelConfigBaseRespVO> result = list.stream().map(i -> BeanUtil.copyProperties(i, MemberUserLevelConfigBaseRespVO.class)).collect(Collectors.toList());
        Long total = memberUserLevelConfigMapper.selectCount();
        return new PageResult<>(result, total, query.getPageSize(), query.getPageNo(), (total + query.getPageSize() - 1) / query.getPageSize());
    }

    @Override
    public Boolean delete(List<Long> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            return true;
        }
        LambdaUpdateWrapper<MemberUserLevelConfigDO> wrapper = Wrappers.lambdaUpdate();
        wrapper.in(MemberUserLevelConfigDO::getId, ids);
        wrapper.set(MemberUserLevelConfigDO::getDeleted, 1);
        return this.update(wrapper);
    }

    @Override
    public Boolean add(MemberUserLevelConfigUpdateVO query) {
        validate(query);
        MemberUserLevelConfigDO memberUserLevelConfigDO = BeanUtil.copyProperties(query, MemberUserLevelConfigDO.class);
        return this.saveOrUpdate(memberUserLevelConfigDO);
    }

    private void validate(MemberUserLevelConfigUpdateVO query) {
        if (query.getLowerCount() > query.getUpperCount()) {
            throw exception(LEVEL_BOUND_RANGE_ERROR);
        }
        LambdaQueryWrapper<MemberUserLevelConfigDO> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(MemberUserLevelConfigDO::getDeleted, 0);
        if (query.getId() != null) {
            wrapper.ne(MemberUserLevelConfigDO::getId, query.getId());
        }
        List<MemberUserLevelConfigDO> list = this.list(wrapper);
        for (MemberUserLevelConfigDO memberUserLevelConfigDO : list) {
            if (query.getLowerCount() >= memberUserLevelConfigDO.getLowerCount() &&  query.getLowerCount() <= memberUserLevelConfigDO.getUpperCount()) {
                throw exception(LEVEL_BOUND_RANGE_CONFLICT);
            }
            if (query.getUpperCount() >= memberUserLevelConfigDO.getLowerCount() &&  query.getUpperCount() <= memberUserLevelConfigDO.getUpperCount()) {
                throw exception(LEVEL_BOUND_RANGE_CONFLICT);
            }
        }
    }

    @Override
    public Boolean updateLevel(MemberUserLevelConfigUpdateVO query) {
        validate(query);
        MemberUserLevelConfigDO memberUserLevelConfigDO = BeanUtil.copyProperties(query, MemberUserLevelConfigDO.class);
        return this.saveOrUpdate(memberUserLevelConfigDO);
    }

    @Override
    public MemberUserLevelConfigBaseRespVO getLevelByCount(Integer scoreCount) {
        List<MemberUserLevelConfigDO> levelConfigList = this.list();
        List<MemberUserLevelConfigDO> sortLevelConfig = levelConfigList.stream()
                .sorted(Comparator.comparing(MemberUserLevelConfigDO::getLowerCount))
                .collect(Collectors.toList());
        MemberUserLevelConfigDO memberUserLevelConfigDO = sortLevelConfig.stream().filter(config -> config.getLowerCount() >= scoreCount && config.getUpperCount() <= scoreCount)
                .findAny().orElse(sortLevelConfig.get(sortLevelConfig.size() - 1));
        return BeanUtil.copyProperties(memberUserLevelConfigDO, MemberUserLevelConfigBaseRespVO.class);
    }
}
