package cn.iocoder.yudao.module.member.service.scoreLog;

import cn.hutool.core.lang.generator.SnowflakeGenerator;
import cn.hutool.json.JSONUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.member.dal.dataobject.scoreLog.MemberUserScoreLogDO;
import cn.iocoder.yudao.module.member.dal.mysql.memberUserScoreLog.MemberUserScoreLogMapper;
import cn.iocoder.yudao.module.member.vo.memberUserScoreLog.MemberUserScoreLogBackVO;
import cn.iocoder.yudao.module.member.vo.memberUserScoreLog.MemberUserScoreLogCreateReq;
import cn.iocoder.yudao.module.member.vo.memberUserScoreLog.MemberUserScoreLogQueryVO;
import cn.iocoder.yudao.module.system.service.dict.DictTypeService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.List;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.member.enums.ErrorCodeConstants.MEMBER_ID_IS_NULL;

/**
 * 会员积分日志 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class MemberUserScoreLogServiceImpl extends AbstractService<MemberUserScoreLogMapper, MemberUserScoreLogDO> implements MemberUserScoreLogService {

    @Resource
    private MemberUserScoreLogMapper userScoreLogMapper;

    @Resource
    private DictTypeService dictTypeService;

    @Resource
    private SnowflakeGenerator snowflakeGenerator;


    @Override
    public PageResult<MemberUserScoreLogBackVO> getPage(MemberUserScoreLogQueryVO query) {
        int start = (query.getPageNo() - 1) * query.getPageSize();
        int size = query.getPageSize();
        List<MemberUserScoreLogBackVO> list = userScoreLogMapper.getPageRecordList(start, size, query);
        int total = userScoreLogMapper.getPageCount(query);
        return new PageResult<>(list, total, query.getPageSize(), query.getPageNo(), (total + query.getPageSize() - 1) / query.getPageSize());
    }

    @Override
    public Long createScoreLog(MemberUserScoreLogCreateReq createReq) {
        if (createReq.getMemberId() == null) {
            throw exception(MEMBER_ID_IS_NULL);
        }
        MemberUserScoreLogDO userScoreLogDO = new MemberUserScoreLogDO();
        userScoreLogDO.setId(snowflakeGenerator.next());
        userScoreLogDO.setMemberId(createReq.getMemberId());
        userScoreLogDO.setScoreCount(createReq.getScoreCount());
        userScoreLogDO.setOperateType(createReq.getOperateType());
        userScoreLogDO.setSourceType(createReq.getSourceType());
        userScoreLogDO.setRuleId(createReq.getRuleId());
        userScoreLogDO.setUniqueId(createReq.getUniqueId());
        if (createReq.getExtParam() != null) {
            userScoreLogDO.setExtParam(JSONUtil.toJsonStr(createReq.getExtParam()));
        }
        this.saveOrUpdate(userScoreLogDO);
        return userScoreLogDO.getId();
    }

    @Override
    public MemberUserScoreLogDO getByUniqueId(String uniqueId) {
        LambdaQueryWrapper<MemberUserScoreLogDO> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(MemberUserScoreLogDO::getUniqueId, uniqueId);
        return this.selectOne(wrapper);
    }
}
